@@ -7,7 +7,7 @@
 #include <qwt_scale_draw.h>
 #include <qwt_scale_widget.h>
 #include <qwt_legend.h>
-#include <qwt_legend_item.h>
+#include <qwt_legend_label.h>
 #include <qwt_plot_canvas.h>
 #include "cpupiemarker.h"
 #include "cpuplot.h"
@@ -21,7 +21,7 @@ public:
     }
     virtual QwtText label( double v ) const
     {
-        QTime upTime = baseTime.addSecs( ( int )v );
+        QTime upTime = baseTime.addSecs( static_cast<int>( v ) );
         return upTime.toString();
     }
 private:
@@ -73,7 +73,7 @@ public:
         QColor c = color;
         c.setAlpha( 150 );
 
-        setPen( c );
+        setPen( QPen( Qt::NoPen ) );
         setBrush( c );
     }
 };
@@ -84,12 +84,15 @@ CpuPlot::CpuPlot( QWidget *parent ):
 {
     setAutoReplot( false );
 
-    canvas()->setBorderRadius( 10 );
+    QwtPlotCanvas *canvas = new QwtPlotCanvas();
+    canvas->setBorderRadius( 10 );
+
+    setCanvas( canvas );
 
     plotLayout()->setAlignCanvasToScales( true );
 
     QwtLegend *legend = new QwtLegend;
-    legend->setItemMode( QwtLegend::CheckableItem );
+    legend->setDefaultItemMode( QwtLegendData::Checkable );
     insertLegend( legend, QwtPlot::RightLegend );
 
     setAxisTitle( QwtPlot::xBottom, " System Uptime [h:m:s]" );
@@ -156,8 +159,8 @@ CpuPlot::CpuPlot( QWidget *parent ):
 
     ( void )startTimer( 1000 ); // 1 second
 
-    connect( this, SIGNAL( legendChecked( QwtPlotItem *, bool ) ),
-        SLOT( showCurve( QwtPlotItem *, bool ) ) );
+    connect( legend, SIGNAL( checked( const QVariant &, bool, int ) ),
+        SLOT( legendChecked( const QVariant &, bool ) ) );
 }
 
 void CpuPlot::timerEvent( QTimerEvent * )
@@ -194,15 +197,30 @@ void CpuPlot::timerEvent( QTimerEvent * )
     replot();
 }
 
+void CpuPlot::legendChecked( const QVariant &itemInfo, bool on )
+{
+    QwtPlotItem *plotItem = infoToItem( itemInfo );
+    if ( plotItem )
+        showCurve( plotItem, on );
+}
+
 void CpuPlot::showCurve( QwtPlotItem *item, bool on )
 {
     item->setVisible( on );
 
-    QwtLegendItem *legendItem =
-        qobject_cast<QwtLegendItem *>( legend()->find( item ) );
+    QwtLegend *lgd = qobject_cast<QwtLegend *>( legend() );
 
-    if ( legendItem )
-        legendItem->setChecked( on );
+    QList<QWidget *> legendWidgets = 
+        lgd->legendWidgets( itemToInfo( item ) );
+
+    if ( legendWidgets.size() == 1 )
+    {
+        QwtLegendLabel *legendLabel =
+            qobject_cast<QwtLegendLabel *>( legendWidgets[0] );
+
+        if ( legendLabel )
+            legendLabel->setChecked( on );
+    }
 
     replot();
 }
