@@ -10,7 +10,7 @@ CpuStat::CpuStat()
 
 QTime CpuStat::upTime() const
 {
-    QTime t;
+    QTime t( 0, 0, 0 );
     for ( int i = 0; i < NValues; i++ )
         t = t.addSecs( int( procValues[i] / 100 ) );
 
@@ -41,7 +41,11 @@ void CpuStat::statistic( double &user, double &system )
 void CpuStat::lookUp( double values[NValues] ) const
 {
     QFile file( "/proc/stat" );
+#if 1
     if ( !file.open( QIODevice::ReadOnly ) )
+#else
+    if ( true )
+#endif
     {
         static double dummyValues[][NValues] =
         {
