@@ -1,26 +1,30 @@
 #include <qpainter.h>
 #include <qwt_dial_needle.h>
+#include <qwt_round_scale_draw.h>
 #include "speedo_meter.h"
 
 SpeedoMeter::SpeedoMeter( QWidget *parent ):
     QwtDial( parent ),
     d_label( "km/h" )
 {
+    QwtRoundScaleDraw *scaleDraw = new QwtRoundScaleDraw();
+    scaleDraw->setSpacing( 8 );
+    scaleDraw->enableComponent( QwtAbstractScaleDraw::Backbone, false );
+    scaleDraw->setTickLength( QwtScaleDiv::MinorTick, 0 );
+    scaleDraw->setTickLength( QwtScaleDiv::MediumTick, 4 );
+    scaleDraw->setTickLength( QwtScaleDiv::MajorTick, 8 );
+    setScaleDraw( scaleDraw );
+
     setWrapping( false );
     setReadOnly( true );
 
     setOrigin( 135.0 );
     setScaleArc( 0.0, 270.0 );
-    scaleDraw()->setSpacing( 8 );
 
     QwtDialSimpleNeedle *needle = new QwtDialSimpleNeedle(
         QwtDialSimpleNeedle::Arrow, true, Qt::red,
         QColor( Qt::gray ).light( 130 ) );
     setNeedle( needle );
-
-    setScaleComponents(
-        QwtAbstractScaleDraw::Ticks | QwtAbstractScaleDraw::Labels );
-    setScaleTicks( 0, 4, 8 );
 }
 
 void SpeedoMeter::setLabel( const QString &label )
