@@ -1,7 +1,8 @@
-#include <qevent.h>
+#include "scalepicker.h"
 #include <qwt_plot.h>
 #include <qwt_scale_widget.h>
-#include "scalepicker.h"
+#include <qevent.h>
+#include <qmath.h>
 
 ScalePicker::ScalePicker( QwtPlot *plot ):
     QObject( plot )
@@ -21,7 +22,7 @@ bool ScalePicker::eventFilter( QObject *object, QEvent *event )
         QwtScaleWidget *scaleWidget = qobject_cast<QwtScaleWidget *>( object );
         if ( scaleWidget )
         {
-            QMouseEvent *mouseEvent = ( QMouseEvent * ) event;
+            QMouseEvent *mouseEvent = static_cast<QMouseEvent *>( event );
             mouseClicked( scaleWidget, mouseEvent->pos() );
 
             return true;
@@ -82,7 +83,7 @@ void ScalePicker::mouseClicked( const QwtScaleWidget *scale, const QPoint &pos )
 QRect ScalePicker::scaleRect( const QwtScaleWidget *scale ) const
 {
     const int bld = scale->margin();
-    const int mjt = scale->scaleDraw()->maxTickLength();
+    const int mjt = qCeil( scale->scaleDraw()->maxTickLength() );
     const int sbd = scale->startBorderDist();
     const int ebd = scale->endBorderDist();
 
