@@ -9,8 +9,6 @@
 #include <qwt_plot_renderer.h>
 #include <qwt_plot_grid.h>
 #include <qwt_plot_canvas.h>
-#include <qfiledialog.h>
-#include <qimagewriter.h>
 
 class RasterData: public QwtMatrixRasterData
 {
@@ -56,9 +54,13 @@ public:
 Plot::Plot( QWidget *parent ):
     QwtPlot( parent )
 {
+    QwtPlotCanvas *canvas = new QwtPlotCanvas();
+    canvas->setBorderRadius( 10 );
+    setCanvas( canvas );
+
 #if 0
     QwtPlotGrid *grid = new QwtPlotGrid();
-    grid->setPen( QPen( Qt::DotLine ) );
+    grid->setPen( Qt::DotLine );
     grid->attach( this );
 #endif
 
@@ -87,60 +89,24 @@ Plot::Plot( QWidget *parent ):
     setAxisScale( QwtPlot::yLeft, 0.0, 3.0 );
     setAxisMaxMinor( QwtPlot::yLeft, 0 );
 
-    QwtPlotMagnifier *magnifier = new QwtPlotMagnifier( canvas() );
+    QwtPlotMagnifier *magnifier = new QwtPlotMagnifier( canvas );
     magnifier->setAxisEnabled( QwtPlot::yRight, false );
 
-    QwtPlotPanner *panner = new QwtPlotPanner( canvas() );
+    QwtPlotPanner *panner = new QwtPlotPanner( canvas );
     panner->setAxisEnabled( QwtPlot::yRight, false );
-
-    canvas()->setBorderRadius( 10 );
 }
 
 void Plot::exportPlot()
 {
-    QString fileName = "rasterview.pdf";
-
-#ifndef QT_NO_FILEDIALOG
-    const QList<QByteArray> imageFormats =
-        QImageWriter::supportedImageFormats();
-
-    QStringList filter;
-    filter += "PDF Documents (*.pdf)";
-#ifndef QWT_NO_SVG
-    filter += "SVG Documents (*.svg)";
-#endif
-    filter += "Postscript Documents (*.ps)";
-
-    if ( imageFormats.size() > 0 )
-    {
-        QString imageFilter( "Images (" );
-        for ( int i = 0; i < imageFormats.size(); i++ )
-        {
-            if ( i > 0 )
-                imageFilter += " ";
-            imageFilter += "*.";
-            imageFilter += imageFormats[i];
-        }
-        imageFilter += ")";
-
-        filter += imageFilter;
-    }
-
-    fileName = QFileDialog::getSaveFileName(
-        this, "Export File Name", fileName,
-        filter.join( ";;" ), NULL, QFileDialog::DontConfirmOverwrite );
-#endif
-    if ( !fileName.isEmpty() )
-    {
-        QwtPlotRenderer renderer;
-        renderer.renderDocument( this, fileName, QSizeF( 300, 200 ), 85 );
-    }
+    QwtPlotRenderer renderer;
+    renderer.exportTo( this, "rasterview.pdf" );
 }
 
 void Plot::setResampleMode( int mode )
 {
-    RasterData *data = ( RasterData * )d_spectrogram->data();
-    data->setResampleMode( ( QwtMatrixRasterData::ResampleMode ) mode );
+    RasterData *data = static_cast<RasterData *>( d_spectrogram->data() );
+    data->setResampleMode(
+        static_cast<QwtMatrixRasterData::ResampleMode>( mode ) );
 
     replot();
 }
