@@ -128,7 +128,7 @@ double ScrollBar::maxSliderValue() const
 int ScrollBar::mapToTick( double v ) const
 {
     const double pos = ( v - d_minBase ) / ( d_maxBase - d_minBase ) * d_baseTicks;
-    return ( int ) pos;
+    return static_cast<int>( pos );
 }
 
 double ScrollBar::mapFromTick( int tick ) const
