@@ -24,6 +24,7 @@ public:
     Settings()
     {
         grid.pen = Qt::NoPen;
+        grid.pen.setCosmetic( true );
 
         curve.brush = Qt::NoBrush;
         curve.numPoints = 1000;
@@ -31,10 +32,13 @@ public:
         curve.paintAttributes = 0;
         curve.renderHint = 0;
         curve.lineSplitting = true;
-        
+
         canvas.useBackingStore = false;
         canvas.paintOnScreen = false;
         canvas.immediatePaint = true;
+#ifndef QWT_NO_OPENGL
+        canvas.openGL = false;
+#endif
 
         updateType = RepaintCanvas;
         updateInterval = 20;
@@ -61,6 +65,10 @@ public:
         bool useBackingStore;
         bool paintOnScreen;
         bool immediatePaint;
+
+#ifndef QWT_NO_OPENGL
+        bool openGL;
+#endif
     } canvas;
 
     UpdateType updateType;
