@@ -1,13 +1,15 @@
-#include <stdlib.h>
-#include <qpen.h>
+#include "tvplot.h"
 #include <qwt_plot_layout.h>
+#include <qwt_plot_canvas.h>
+#include <qwt_plot_renderer.h>
 #include <qwt_legend.h>
-#include <qwt_legend_item.h>
+#include <qwt_legend_label.h>
 #include <qwt_plot_grid.h>
 #include <qwt_plot_histogram.h>
 #include <qwt_column_symbol.h>
 #include <qwt_series_data.h>
-#include "tvplot.h"
+#include <qpen.h>
+#include <stdlib.h>
 
 class Histogram: public QwtPlotHistogram
 {
@@ -26,19 +28,11 @@ Histogram::Histogram( const QString &title, const QColor &symbolColor ):
     setColor( symbolColor );
 }
 
-void Histogram::setColor( const QColor &symbolColor )
+void Histogram::setColor( const QColor &color )
 {
-    QColor color = symbolColor;
-    color.setAlpha( 180 );
-
-    setPen( QPen( Qt::black ) );
-    setBrush( QBrush( color ) );
-
-    QwtColumnSymbol *symbol = new QwtColumnSymbol( QwtColumnSymbol::Box );
-    symbol->setFrameStyle( QwtColumnSymbol::Raised );
-    symbol->setLineWidth( 2 );
-    symbol->setPalette( QPalette( color ) );
-    setSymbol( symbol );
+    QColor c = color;
+    c.setAlpha( 180 );
+    setBrush( QBrush( c ) );
 }
 
 void Histogram::setValues( uint numValues, const double *values )
@@ -60,20 +54,24 @@ TVPlot::TVPlot( QWidget *parent ):
 {
     setTitle( "Watching TV during a weekend" );
 
-    setCanvasBackground( QColor( Qt::gray ) );
+    QwtPlotCanvas *canvas = new QwtPlotCanvas();
+    canvas->setPalette( Qt::gray );
+    canvas->setBorderRadius( 10 );
+    setCanvas( canvas );
+
     plotLayout()->setAlignCanvasToScales( true );
 
     setAxisTitle( QwtPlot::yLeft, "Number of People" );
     setAxisTitle( QwtPlot::xBottom, "Number of Hours" );
 
     QwtLegend *legend = new QwtLegend;
-    legend->setItemMode( QwtLegend::CheckableItem );
+    legend->setDefaultItemMode( QwtLegendData::Checkable );
     insertLegend( legend, QwtPlot::RightLegend );
 
     populate();
 
-    connect( this, SIGNAL( legendChecked( QwtPlotItem *, bool ) ),
-        SLOT( showItem( QwtPlotItem *, bool ) ) );
+    connect( legend, SIGNAL( checked( const QVariant &, bool, int ) ),
+        SLOT( showItem( const QVariant &, bool ) ) );
 
     replot(); // creating the legend items
 
@@ -82,15 +80,19 @@ TVPlot::TVPlot( QWidget *parent ):
     {
         if ( i == 0 )
         {
-            QwtLegendItem *legendItem =
-                qobject_cast<QwtLegendItem *>( legend->find( items[i] ) );
-            if ( legendItem )
-                legendItem->setChecked( true );
+            const QVariant itemInfo = itemToInfo( items[i] );
+
+            QwtLegendLabel *legendLabel =
+                qobject_cast<QwtLegendLabel *>( legend->legendWidget( itemInfo ) );
+            if ( legendLabel )
+                legendLabel->setChecked( true );
 
             items[i]->setVisible( true );
         }
         else
+        {
             items[i]->setVisible( false );
+        }
     }
 
     setAutoReplot( true );
@@ -103,7 +105,7 @@ void TVPlot::populate()
     grid->enableY( true );
     grid->enableXMin( false );
     grid->enableYMin( false );
-    grid->setMajPen( QPen( Qt::black, 0, Qt::DotLine ) );
+    grid->setMajorPen( Qt::black, 0, Qt::DotLine );
     grid->attach( this );
 
     const double juneValues[] = { 7, 19, 24, 32, 10, 5, 3 };
@@ -120,8 +122,48 @@ void TVPlot::populate()
     histogramNovember->attach( this );
 }
 
-void TVPlot::showItem( QwtPlotItem *item, bool on )
+void TVPlot::exportPlot()
 {
-    item->setVisible( on );
+    QwtPlotRenderer renderer;
+    renderer.exportTo( this, "tvplot.pdf" );
+}
+
+void TVPlot::setMode( int mode )
+{
+    QwtPlotItemList items = itemList( QwtPlotItem::Rtti_PlotHistogram );
+
+    for ( int i = 0; i < items.size(); i++ )
+    {
+        QwtPlotHistogram *histogram = static_cast<QwtPlotHistogram *>( items[i] );
+        if ( mode < 3 )
+        {
+            histogram->setStyle( static_cast<QwtPlotHistogram::HistogramStyle>( mode ) );
+            histogram->setSymbol( NULL );
+
+            QPen pen( Qt::black, 0 );
+            if ( mode == QwtPlotHistogram::Lines )
+                pen.setBrush( histogram->brush() );
+
+            histogram->setPen( pen );
+        }
+        else
+        {
+            histogram->setStyle( QwtPlotHistogram::Columns );
+
+            QwtColumnSymbol *symbol = new QwtColumnSymbol( QwtColumnSymbol::Box );
+            symbol->setFrameStyle( QwtColumnSymbol::Raised );
+            symbol->setLineWidth( 2 );
+            symbol->setPalette( QPalette( histogram->brush().color() ) );
+
+            histogram->setSymbol( symbol );
+        }
+    }
+}
+
+void TVPlot::showItem( const QVariant &itemInfo, bool on )
+{
+    QwtPlotItem *plotItem = infoToItem( itemInfo );
+    if ( plotItem )
+        plotItem->setVisible( on );
 }
 
