@@ -10,9 +10,8 @@
 #include "qwt_plot_layout.h"
 #include "qwt_text.h"
 #include "qwt_text_label.h"
-#include "qwt_plot_canvas.h"
 #include "qwt_scale_widget.h"
-#include "qwt_legend.h"
+#include "qwt_abstract_legend.h"
 #include <qscrollbar.h>
 #include <qmath.h>
 
@@ -24,8 +23,8 @@ public:
     struct t_legendData
     {
         int frameWidth;
-        int vScrollBarWidth;
-        int hScrollBarHeight;
+        int hScrollExtent;
+        int vScrollExtent;
         QSize hint;
     } legend;
 
@@ -35,6 +34,12 @@ public:
         int frameWidth;
     } title;
 
+    struct t_footerData
+    {
+        QwtText text;
+        int frameWidth;
+    } footer;
+
     struct t_scaleData
     {
         bool isEnabled;
@@ -49,36 +54,33 @@ public:
 
     struct t_canvasData
     {
-        int frameWidth;
+        int contentsMargins[ QwtPlot::axisCnt ];
+
     } canvas;
 };
 
 /*
   Extract all layout relevant data from the plot components
 */
-
 void QwtPlotLayout::LayoutData::init( const QwtPlot *plot, const QRectF &rect )
 {
     // legend
 
-    if ( plot->plotLayout()->legendPosition() != QwtPlot::ExternalLegend
-        && plot->legend() )
+    if ( plot->legend() )
     {
         legend.frameWidth = plot->legend()->frameWidth();
-        legend.vScrollBarWidth =
-            plot->legend()->verticalScrollBar()->sizeHint().width();
-        legend.hScrollBarHeight =
-            plot->legend()->horizontalScrollBar()->sizeHint().height();
+        legend.hScrollExtent =
+            plot->legend()->scrollExtent( Qt::Horizontal );
+        legend.vScrollExtent =
+            plot->legend()->scrollExtent( Qt::Vertical );
 
         const QSize hint = plot->legend()->sizeHint();
 
-        int w = qMin( hint.width(), qFloor( rect.width() ) );
-        int h = plot->legend()->heightForWidth( w );
-        if ( h == 0 )
-            h = hint.height();
+        const int w = qMin( hint.width(), qFloor( rect.width() ) );
 
-        if ( h > rect.height() )
-            w += legend.vScrollBarWidth;
+        int h = plot->legend()->heightForWidth( w );
+        if ( h <= 0 )
+            h = hint.height();
 
         legend.hint = QSize( w, h );
     }
@@ -98,6 +100,21 @@ void QwtPlotLayout::LayoutData::init( const QwtPlot *plot, const QRectF &rect )
         title.frameWidth = plot->titleLabel()->frameWidth();
     }
 
+    // footer
+
+    footer.frameWidth = 0;
+    footer.text = QwtText();
+
+    if ( plot->footerLabel() )
+    {
+        const QwtTextLabel *label = plot->footerLabel();
+        footer.text = label->text();
+        if ( !( footer.text.testPaintAttribute( QwtText::PaintUsingTextFont ) ) )
+            footer.text.setFont( label->font() );
+
+        footer.frameWidth = plot->footerLabel()->frameWidth();
+    }
+
     // scales
 
     for ( int axis = 0; axis < QwtPlot::axisCnt; axis++ )
@@ -146,19 +163,23 @@ void QwtPlotLayout::LayoutData::init( const QwtPlot *plot, const QRectF &rect )
 
     // canvas
 
-    canvas.frameWidth = plot->canvas()->frameWidth();
+    plot->canvas()->getContentsMargins( 
+        &canvas.contentsMargins[ QwtPlot::yLeft ], 
+        &canvas.contentsMargins[ QwtPlot::xTop ],
+        &canvas.contentsMargins[ QwtPlot::yRight ],
+        &canvas.contentsMargins[ QwtPlot::xBottom ] );
 }
 
 class QwtPlotLayout::PrivateData
 {
 public:
     PrivateData():
-        spacing( 5 ),
-        alignCanvasToScales( false )
+        spacing( 5 )
     {
     }
 
     QRectF titleRect;
+    QRectF footerRect;
     QRectF legendRect;
     QRectF scaleRect[QwtPlot::axisCnt];
     QRectF canvasRect;
@@ -169,7 +190,7 @@ public:
     double legendRatio;
     unsigned int spacing;
     unsigned int canvasMargin[QwtPlot::axisCnt];
-    bool alignCanvasToScales;
+    bool alignCanvasToScales[QwtPlot::axisCnt];
 };
 
 /*!
@@ -182,6 +203,7 @@ QwtPlotLayout::QwtPlotLayout()
 
     setLegendPosition( QwtPlot::BottomLegend );
     setCanvasMargin( 4 );
+    setAlignCanvasToScales( false );
 
     invalidate();
 }
@@ -202,7 +224,7 @@ QwtPlotLayout::~QwtPlotLayout()
               -1 means margin at all borders.
   \sa canvasMargin()
 
-  \warning The margin will have no effect when alignCanvasToScales is true
+  \warning The margin will have no effect when alignCanvasToScale() is true
 */
 
 void QwtPlotLayout::setCanvasMargin( int margin, int axis )
@@ -220,32 +242,51 @@ void QwtPlotLayout::setCanvasMargin( int margin, int axis )
 }
 
 /*!
+    \param axisId Axis index
     \return Margin around the scale tick borders
     \sa setCanvasMargin()
 */
-int QwtPlotLayout::canvasMargin( int axis ) const
+int QwtPlotLayout::canvasMargin( int axisId ) const
 {
-    if ( axis < 0 || axis >= QwtPlot::axisCnt )
+    if ( axisId < 0 || axisId >= QwtPlot::axisCnt )
         return 0;
 
-    return d_data->canvasMargin[axis];
+    return d_data->canvasMargin[axisId];
+}
+
+/*!
+  \brief Set the align-canvas-to-axis-scales flag for all axes
+
+  \param on True/False
+  \sa setAlignCanvasToScale(), alignCanvasToScale()
+*/
+void QwtPlotLayout::setAlignCanvasToScales( bool on )
+{
+    for ( int axis = 0; axis < QwtPlot::axisCnt; axis++ )
+        d_data->alignCanvasToScales[axis] = on;
 }
 
 /*!
   Change the align-canvas-to-axis-scales setting. The canvas may:
+
   - extend beyond the axis scale ends to maximize its size,
   - align with the axis scale ends to control its size.
 
-  \param alignCanvasToScales New align-canvas-to-axis-scales setting
+  The axisId parameter is somehow confusing as it identifies a border
+  of the plot and not the axes, that are aligned. F.e when QwtPlot::yLeft
+  is set, the left end of the the x-axes ( QwtPlot::xTop, QwtPlot::xBottom )
+  is aligned.
 
-  \sa setCanvasMargin()
-  \note In this context the term 'scale' means the backbone of a scale.
-  \warning In case of alignCanvasToScales == true canvasMargin will have
-           no effect
+  \param axisId Axis index
+  \param on New align-canvas-to-axis-scales setting
+
+  \sa setCanvasMargin(), alignCanvasToScale(), setAlignCanvasToScales()
+  \warning In case of on == true canvasMargin() will have no effect
 */
-void QwtPlotLayout::setAlignCanvasToScales( bool alignCanvasToScales )
+void QwtPlotLayout::setAlignCanvasToScale( int axisId, bool on )
 {
-    d_data->alignCanvasToScales = alignCanvasToScales;
+    if ( axisId >= 0 && axisId < QwtPlot::axisCnt )
+        d_data->alignCanvasToScales[axisId] = on;
 }
 
 /*!
@@ -253,21 +294,24 @@ void QwtPlotLayout::setAlignCanvasToScales( bool alignCanvasToScales )
   - extend beyond the axis scale ends to maximize its size
   - align with the axis scale ends to control its size.
 
+  \param axisId Axis index
   \return align-canvas-to-axis-scales setting
-  \sa setAlignCanvasToScales, setCanvasMargin()
-  \note In this context the term 'scale' means the backbone of a scale.
+  \sa setAlignCanvasToScale(), setAlignCanvasToScale(), setCanvasMargin()
 */
-bool QwtPlotLayout::alignCanvasToScales() const
+bool QwtPlotLayout::alignCanvasToScale( int axisId ) const
 {
-    return d_data->alignCanvasToScales;
+    if ( axisId < 0 || axisId >= QwtPlot::axisCnt )
+        return false;
+
+    return d_data->alignCanvasToScales[ axisId ];
 }
 
 /*!
   Change the spacing of the plot. The spacing is the distance
   between the plot components.
 
-  \param spacing new spacing
-  \sa setMargin(), spacing()
+  \param spacing New spacing
+  \sa setCanvasMargin(), spacing()
 */
 void QwtPlotLayout::setSpacing( int spacing )
 {
@@ -275,7 +319,7 @@ void QwtPlotLayout::setSpacing( int spacing )
 }
 
 /*!
-  \return spacing
+  \return Spacing
   \sa margin(), setSpacing()
 */
 int QwtPlotLayout::spacing() const
@@ -286,8 +330,8 @@ int QwtPlotLayout::spacing() const
 /*!
   \brief Specify the position of the legend
   \param pos The legend's position.
-  \param ratio Ratio between legend and the bounding rect
-               of title, canvas and axes. The legend will be shrinked
+  \param ratio Ratio between legend and the bounding rectangle
+               of title, footer, canvas and axes. The legend will be shrunk
                if it would need more space than the given ratio.
                The ratio is limited to ]0.0 .. 1.0]. In case of <= 0.0
                it will be reset to the default ratio.
@@ -317,9 +361,6 @@ void QwtPlotLayout::setLegendPosition( QwtPlot::LegendPosition pos, double ratio
             d_data->legendRatio = ratio;
             d_data->legendPos = pos;
             break;
-        case QwtPlot::ExternalLegend:
-            d_data->legendRatio = ratio; // meaningless
-            d_data->legendPos = pos;
         default:
             break;
     }
@@ -350,8 +391,8 @@ QwtPlot::LegendPosition QwtPlotLayout::legendPosition() const
 
 /*!
   Specify the relative size of the legend in the plot
-  \param ratio Ratio between legend and the bounding rect
-               of title, canvas and axes. The legend will be shrinked
+  \param ratio Ratio between legend and the bounding rectangle
+               of title, footer, canvas and axes. The legend will be shrunk
                if it would need more space than the given ratio.
                The ratio is limited to ]0.0 .. 1.0]. In case of <= 0.0
                it will be reset to the default ratio.
@@ -371,30 +412,97 @@ double QwtPlotLayout::legendRatio() const
     return d_data->legendRatio;
 }
 
+/*!
+  \brief Set the geometry for the title
+
+  This method is intended to be used from derived layouts
+  overloading activate()
+
+  \sa titleRect(), activate()
+ */
+void QwtPlotLayout::setTitleRect( const QRectF &rect )
+{
+    d_data->titleRect = rect;
+}
+
 /*!
   \return Geometry for the title
   \sa activate(), invalidate()
 */
-const QRectF &QwtPlotLayout::titleRect() const
+QRectF QwtPlotLayout::titleRect() const
 {
     return d_data->titleRect;
 }
 
+/*!
+  \brief Set the geometry for the footer
+
+  This method is intended to be used from derived layouts
+  overloading activate()
+
+  \sa footerRect(), activate()
+ */
+void QwtPlotLayout::setFooterRect( const QRectF &rect )
+{
+    d_data->footerRect = rect;
+}
+
+/*!
+  \return Geometry for the footer
+  \sa activate(), invalidate()
+*/
+QRectF QwtPlotLayout::footerRect() const
+{
+    return d_data->footerRect;
+}
+
+/*!
+  \brief Set the geometry for the legend
+
+  This method is intended to be used from derived layouts
+  overloading activate()
+
+  \param rect Rectangle for the legend
+
+  \sa legendRect(), activate()
+ */
+void QwtPlotLayout::setLegendRect( const QRectF &rect )
+{
+    d_data->legendRect = rect;
+}
+
 /*!
   \return Geometry for the legend
   \sa activate(), invalidate()
 */
-const QRectF &QwtPlotLayout::legendRect() const
+QRectF QwtPlotLayout::legendRect() const
 {
     return d_data->legendRect;
 }
 
+/*!
+  \brief Set the geometry for an axis
+
+  This method is intended to be used from derived layouts
+  overloading activate()
+
+  \param axis Axis index
+  \param rect Rectangle for the scale
+
+  \sa scaleRect(), activate()
+ */
+void QwtPlotLayout::setScaleRect( int axis, const QRectF &rect )
+{
+    if ( axis >= 0 && axis < QwtPlot::axisCnt )
+        d_data->scaleRect[axis] = rect;
+}
+
 /*!
   \param axis Axis index
   \return Geometry for the scale
   \sa activate(), invalidate()
 */
-const QRectF &QwtPlotLayout::scaleRect( int axis ) const
+QRectF QwtPlotLayout::scaleRect( int axis ) const
 {
     if ( axis < 0 || axis >= QwtPlot::axisCnt )
     {
@@ -404,11 +512,24 @@ const QRectF &QwtPlotLayout::scaleRect( int axis ) const
     return d_data->scaleRect[axis];
 }
 
+/*!
+  \brief Set the geometry for the canvas
+
+  This method is intended to be used from derived layouts
+  overloading activate()
+
+  \sa canvasRect(), activate()
+ */
+void QwtPlotLayout::setCanvasRect( const QRectF &rect )
+{
+    d_data->canvasRect = rect;
+}
+
 /*!
   \return Geometry for the canvas
   \sa activate(), invalidate()
 */
-const QRectF &QwtPlotLayout::canvasRect() const
+QRectF QwtPlotLayout::canvasRect() const
 {
     return d_data->canvasRect;
 }
@@ -419,13 +540,17 @@ const QRectF &QwtPlotLayout::canvasRect() const
 */
 void QwtPlotLayout::invalidate()
 {
-    d_data->titleRect = d_data->legendRect = d_data->canvasRect = QRect();
+    d_data->titleRect = d_data->footerRect
+        = d_data->legendRect = d_data->canvasRect = QRect();
+
     for ( int axis = 0; axis < QwtPlot::axisCnt; axis++ )
         d_data->scaleRect[axis] = QRect();
 }
 
 /*!
-  \brief Return a minimum size hint
+  \return Minimum size hint
+  \param plot Plot widget
+
   \sa QwtPlot::minimumSizeHint()
 */
 
@@ -448,6 +573,9 @@ QSize QwtPlotLayout::minimumSizeHint( const QwtPlot *plot ) const
 
     int canvasBorder[QwtPlot::axisCnt];
 
+    int fw;
+    plot->canvas()->getContentsMargins( &fw, NULL, NULL, NULL );
+
     int axis;
     for ( axis = 0; axis < QwtPlot::axisCnt; axis++ )
     {
@@ -465,9 +593,7 @@ QSize QwtPlotLayout::minimumSizeHint( const QwtPlot *plot ) const
                 sd.tickOffset += qCeil( scl->scaleDraw()->maxTickLength() );
         }
 
-        canvasBorder[axis] = plot->canvas()->frameWidth() +
-            d_data->canvasMargin[axis] + 1;
-
+        canvasBorder[axis] = fw + d_data->canvasMargin[axis] + 1;
     }
 
 
@@ -519,54 +645,64 @@ QSize QwtPlotLayout::minimumSizeHint( const QwtPlot *plot ) const
         }
     }
 
-    const QwtPlotCanvas *canvas = plot->canvas();
+    const QWidget *canvas = plot->canvas();
+
+    int left, top, right, bottom;
+    canvas->getContentsMargins( &left, &top, &right, &bottom );
+
     const QSize minCanvasSize = canvas->minimumSize();
 
     int w = scaleData[QwtPlot::yLeft].w + scaleData[QwtPlot::yRight].w;
     int cw = qMax( scaleData[QwtPlot::xBottom].w, scaleData[QwtPlot::xTop].w )
-        + 2 * ( canvas->frameWidth() + 1 );
+        + left + 1 + right + 1;
     w += qMax( cw, minCanvasSize.width() );
 
     int h = scaleData[QwtPlot::xBottom].h + scaleData[QwtPlot::xTop].h;
     int ch = qMax( scaleData[QwtPlot::yLeft].h, scaleData[QwtPlot::yRight].h )
-        + 2 * ( canvas->frameWidth() + 1 );
+        + top + 1 + bottom + 1;
     h += qMax( ch, minCanvasSize.height() );
 
-    const QwtTextLabel *title = plot->titleLabel();
-    if ( title && !title->text().isEmpty() )
+    const QwtTextLabel *labels[2];
+    labels[0] = plot->titleLabel();
+    labels[1] = plot->footerLabel();
+
+    for ( int i = 0; i < 2; i++ )
     {
-        // If only QwtPlot::yLeft or QwtPlot::yRight is showing,
-        // we center on the plot canvas.
-        const bool centerOnCanvas = !( plot->axisEnabled( QwtPlot::yLeft )
-            && plot->axisEnabled( QwtPlot::yRight ) );
-
-        int titleW = w;
-        if ( centerOnCanvas )
+        const QwtTextLabel *label   = labels[i];
+        if ( label && !label->text().isEmpty() )
         {
-            titleW -= scaleData[QwtPlot::yLeft].w
-                + scaleData[QwtPlot::yRight].w;
-        }
+            // If only QwtPlot::yLeft or QwtPlot::yRight is showing,
+            // we center on the plot canvas.
+            const bool centerOnCanvas = !( plot->axisEnabled( QwtPlot::yLeft )
+                && plot->axisEnabled( QwtPlot::yRight ) );
 
-        int titleH = title->heightForWidth( titleW );
-        if ( titleH > titleW ) // Compensate for a long title
-        {
-            w = titleW = titleH;
+            int labelW = w;
             if ( centerOnCanvas )
             {
-                w += scaleData[QwtPlot::yLeft].w
+                labelW -= scaleData[QwtPlot::yLeft].w
                     + scaleData[QwtPlot::yRight].w;
             }
 
-            titleH = title->heightForWidth( titleW );
+            int labelH = label->heightForWidth( labelW );
+            if ( labelH > labelW ) // Compensate for a long title
+            {
+                w = labelW = labelH;
+                if ( centerOnCanvas )
+                {
+                    w += scaleData[QwtPlot::yLeft].w
+                        + scaleData[QwtPlot::yRight].w;
+                }
+
+                labelH = label->heightForWidth( labelW );
+            }
+            h += labelH + d_data->spacing;
         }
-        h += titleH + d_data->spacing;
     }
 
     // Compute the legend contribution
 
-    const QwtLegend *legend = plot->legend();
-    if ( d_data->legendPos != QwtPlot::ExternalLegend
-        && legend && !legend->isEmpty() )
+    const QwtAbstractLegend *legend = plot->legend();
+    if ( legend && !legend->isEmpty() )
     {
         if ( d_data->legendPos == QwtPlot::LeftLegend
             || d_data->legendPos == QwtPlot::RightLegend )
@@ -578,7 +714,7 @@ QSize QwtPlotLayout::minimumSizeHint( const QwtPlot *plot ) const
                 w += d_data->spacing;
 
             if ( legendH > h )
-                legendW += legend->verticalScrollBar()->sizeHint().width();
+                legendW += legend->scrollExtent( Qt::Horizontal );
 
             if ( d_data->legendRatio < 1.0 )
                 legendW = qMin( legendW, int( w / ( 1.0 - d_data->legendRatio ) ) );
@@ -605,8 +741,10 @@ QSize QwtPlotLayout::minimumSizeHint( const QwtPlot *plot ) const
 
 /*!
   Find the geometry for the legend
+
   \param options Options how to layout the legend
   \param rect Rectangle where to place the legend
+
   \return Geometry for the legend
   \sa Options
 */
@@ -632,14 +770,14 @@ QRectF QwtPlotLayout::layoutLegend( Options options,
                 // The legend will need additional
                 // space for the vertical scrollbar.
 
-                dim += d_data->layoutData.legend.vScrollBarWidth;
+                dim += d_data->layoutData.legend.hScrollExtent;
             }
         }
     }
     else
     {
         dim = qMin( hint.height(), int( rect.height() * d_data->legendRatio ) );
-        dim = qMax( dim, d_data->layoutData.legend.hScrollBarHeight );
+        dim = qMax( dim, d_data->layoutData.legend.vScrollExtent );
     }
 
     QRectF legendRect = rect;
@@ -659,8 +797,6 @@ QRectF QwtPlotLayout::layoutLegend( Options options,
             legendRect.setY( rect.bottom() - dim );
             legendRect.setHeight( dim );
             break;
-        case QwtPlot::ExternalLegend:
-            break;
     }
 
     return legendRect;
@@ -668,8 +804,10 @@ QRectF QwtPlotLayout::layoutLegend( Options options,
 
 /*!
   Align the legend to the canvas
+
   \param canvasRect Geometry of the canvas
   \param legendRect Maximum geometry for the legend
+
   \return Geometry for the aligned legend
 */
 QRectF QwtPlotLayout::alignLegend( const QRectF &canvasRect,
@@ -703,16 +841,17 @@ QRectF QwtPlotLayout::alignLegend( const QRectF &canvasRect,
   of their widgets in orientation of the text.
 
   \param options Options how to layout the legend
-  \param rect Bounding rect for title, axes and canvas.
+  \param rect Bounding rectangle for title, footer, axes and canvas.
   \param dimTitle Expanded height of the title widget
+  \param dimFooter Expanded height of the footer widget
   \param dimAxis Expanded heights of the axis in axis orientation.
 
   \sa Options
 */
-void QwtPlotLayout::expandLineBreaks( int options, const QRectF &rect,
-    int &dimTitle, int dimAxis[QwtPlot::axisCnt] ) const
+void QwtPlotLayout::expandLineBreaks( Options options, const QRectF &rect,
+    int &dimTitle, int &dimFooter, int dimAxis[QwtPlot::axisCnt] ) const
 {
-    dimTitle = 0;
+    dimTitle = dimFooter = 0;
     for ( int axis = 0; axis < QwtPlot::axisCnt; axis++ )
         dimAxis[axis] = 0;
 
@@ -720,10 +859,11 @@ void QwtPlotLayout::expandLineBreaks( int options, const QRectF &rect,
     for ( int axis = 0; axis < QwtPlot::axisCnt; axis++ )
     {
         backboneOffset[axis] = 0;
-        if ( !d_data->alignCanvasToScales )
-            backboneOffset[axis] += d_data->canvasMargin[axis];
         if ( !( options & IgnoreFrames ) )
-            backboneOffset[axis] += d_data->layoutData.canvas.frameWidth;
+            backboneOffset[axis] += d_data->layoutData.canvas.contentsMargins[ axis ];
+
+        if ( !d_data->alignCanvasToScales[axis] )
+            backboneOffset[axis] += d_data->canvasMargin[axis];
     }
 
     bool done = false;
@@ -739,7 +879,8 @@ void QwtPlotLayout::expandLineBreaks( int options, const QRectF &rect,
         // axis what might result in a line break of a horizontal
         // axis ... . So we loop as long until no size changes.
 
-        if ( !d_data->layoutData.title.text.isEmpty() )
+        if ( !( ( options & IgnoreTitle ) ||
+            d_data->layoutData.title.text.isEmpty() ) )
         {
             double w = rect.width();
 
@@ -761,6 +902,29 @@ void QwtPlotLayout::expandLineBreaks( int options, const QRectF &rect,
             }
         }
 
+        if ( !( ( options & IgnoreFooter ) ||
+            d_data->layoutData.footer.text.isEmpty() ) )
+        {
+            double w = rect.width();
+
+            if ( d_data->layoutData.scale[QwtPlot::yLeft].isEnabled
+                != d_data->layoutData.scale[QwtPlot::yRight].isEnabled )
+            {
+                // center to the canvas
+                w -= dimAxis[QwtPlot::yLeft] + dimAxis[QwtPlot::yRight];
+            }
+
+            int d = qCeil( d_data->layoutData.footer.text.heightForWidth( w ) );
+            if ( !( options & IgnoreFrames ) )
+                d += 2 * d_data->layoutData.footer.frameWidth;
+
+            if ( d > dimFooter )
+            {
+                dimFooter = d;
+                done = false;
+            }
+        }
+
         for ( int axis = 0; axis < QwtPlot::axisCnt; axis++ )
         {
             const struct LayoutData::t_scaleData &scaleData =
@@ -833,20 +997,31 @@ void QwtPlotLayout::expandLineBreaks( int options, const QRectF &rect,
   Align the ticks of the axis to the canvas borders using
   the empty corners.
 
+  \param options Layout options
+  \param canvasRect Geometry of the canvas ( IN/OUT )
+  \param scaleRect Geometries of the scales ( IN/OUT )
+
   \sa Options
 */
 
-void QwtPlotLayout::alignScales( int options,
+void QwtPlotLayout::alignScales( Options options,
     QRectF &canvasRect, QRectF scaleRect[QwtPlot::axisCnt] ) const
 {
     int backboneOffset[QwtPlot::axisCnt];
     for ( int axis = 0; axis < QwtPlot::axisCnt; axis++ )
     {
         backboneOffset[axis] = 0;
-        if ( !d_data->alignCanvasToScales )
+
+        if ( !d_data->alignCanvasToScales[axis] )
+        {
             backboneOffset[axis] += d_data->canvasMargin[axis];
+        }
+
         if ( !( options & IgnoreFrames ) )
-            backboneOffset[axis] += d_data->layoutData.canvas.frameWidth;
+        {
+            backboneOffset[axis] += 
+                d_data->layoutData.canvas.contentsMargins[axis];
+        }
     }
 
     for ( int axis = 0; axis < QwtPlot::axisCnt; axis++ )
@@ -868,7 +1043,7 @@ void QwtPlotLayout::alignScales( int options,
             if ( leftScaleRect.isValid() )
             {
                 const double dx = leftOffset + leftScaleRect.width();
-                if ( d_data->alignCanvasToScales && dx < 0.0 )
+                if ( d_data->alignCanvasToScales[QwtPlot::yLeft] && dx < 0.0 )
                 {
                     /*
                       The axis needs more space than the width
@@ -886,7 +1061,7 @@ void QwtPlotLayout::alignScales( int options,
             }
             else
             {
-                if ( d_data->alignCanvasToScales && leftOffset < 0 )
+                if ( d_data->alignCanvasToScales[QwtPlot::yLeft] && leftOffset < 0 )
                 {
                     canvasRect.setLeft( qMax( canvasRect.left(),
                         axisRect.left() - leftOffset ) );
@@ -905,7 +1080,7 @@ void QwtPlotLayout::alignScales( int options,
             if ( rightScaleRect.isValid() )
             {
                 const double dx = rightOffset + rightScaleRect.width();
-                if ( d_data->alignCanvasToScales && dx < 0 )
+                if ( d_data->alignCanvasToScales[QwtPlot::yRight] && dx < 0 )
                 {
                     /*
                       The axis needs more space than the width
@@ -921,7 +1096,7 @@ void QwtPlotLayout::alignScales( int options,
             }
             else
             {
-                if ( d_data->alignCanvasToScales && rightOffset < 0 )
+                if ( d_data->alignCanvasToScales[QwtPlot::yRight] && rightOffset < 0 )
                 {
                     canvasRect.setRight( qMin( canvasRect.right(),
                         axisRect.right() + rightOffset ) );
@@ -942,7 +1117,7 @@ void QwtPlotLayout::alignScales( int options,
             if ( bottomScaleRect.isValid() )
             {
                 const double dy = bottomOffset + bottomScaleRect.height();
-                if ( d_data->alignCanvasToScales && dy < 0 )
+                if ( d_data->alignCanvasToScales[QwtPlot::xBottom] && dy < 0 )
                 {
                     /*
                       The axis needs more space than the height
@@ -961,7 +1136,7 @@ void QwtPlotLayout::alignScales( int options,
             }
             else
             {
-                if ( d_data->alignCanvasToScales && bottomOffset < 0 )
+                if ( d_data->alignCanvasToScales[QwtPlot::xBottom] && bottomOffset < 0 )
                 {
                     canvasRect.setBottom( qMin( canvasRect.bottom(),
                         axisRect.bottom() + bottomOffset ) );
@@ -979,7 +1154,7 @@ void QwtPlotLayout::alignScales( int options,
             if ( topScaleRect.isValid() )
             {
                 const double dy = topOffset + topScaleRect.height();
-                if ( d_data->alignCanvasToScales && dy < 0 )
+                if ( d_data->alignCanvasToScales[QwtPlot::xTop] && dy < 0 )
                 {
                     /*
                       The axis needs more space than the height
@@ -998,7 +1173,7 @@ void QwtPlotLayout::alignScales( int options,
             }
             else
             {
-                if ( d_data->alignCanvasToScales && topOffset < 0 )
+                if ( d_data->alignCanvasToScales[QwtPlot::xTop] && topOffset < 0 )
                 {
                     canvasRect.setTop( qMax( canvasRect.top(),
                         axisRect.top() - topOffset ) );
@@ -1012,46 +1187,73 @@ void QwtPlotLayout::alignScales( int options,
         }
     }
 
-    if ( d_data->alignCanvasToScales )
+    /*
+      The canvas has been aligned to the scale with largest
+      border distances. Now we have to realign the other scale.
+     */
+
+
+    for ( int axis = 0; axis < QwtPlot::axisCnt; axis++ )
     {
-        /*
-          The canvas has been aligned to the scale with largest
-          border distances. Now we have to realign the other scale.
-         */
+        QRectF &sRect = scaleRect[axis];
 
-        int fw = 0;
-        if ( !( options & IgnoreFrames ) )
-            fw = d_data->layoutData.canvas.frameWidth;
+        if ( !sRect.isValid() )
+            continue;
 
-        for ( int axis = 0; axis < QwtPlot::axisCnt; axis++ )
+        if ( axis == QwtPlot::xBottom || axis == QwtPlot::xTop )
         {
-            if ( !scaleRect[axis].isValid() )
-                continue;
+            if ( d_data->alignCanvasToScales[QwtPlot::yLeft] )
+            {
+                double y = canvasRect.left() - d_data->layoutData.scale[axis].start;
+                if ( !( options & IgnoreFrames ) )
+                    y += d_data->layoutData.canvas.contentsMargins[ QwtPlot::yLeft ];
 
-            if ( axis == QwtPlot::xBottom || axis == QwtPlot::xTop )
+                sRect.setLeft( y );
+            }
+            if ( d_data->alignCanvasToScales[QwtPlot::yRight] )
             {
-                scaleRect[axis].setLeft( canvasRect.left() + fw
-                    - d_data->layoutData.scale[axis].start );
-                scaleRect[axis].setRight( canvasRect.right() - fw - 1
-                    + d_data->layoutData.scale[axis].end );
-            }   
-            else
+                double y = canvasRect.right() - 1 + d_data->layoutData.scale[axis].end;
+                if ( !( options & IgnoreFrames ) )
+                    y -= d_data->layoutData.canvas.contentsMargins[ QwtPlot::yRight ];
+
+                sRect.setRight( y );
+            }
+
+            if ( d_data->alignCanvasToScales[ axis ] )
             {
-                scaleRect[axis].setTop( canvasRect.top() + fw
-                    - d_data->layoutData.scale[axis].start );
-                scaleRect[axis].setBottom( canvasRect.bottom() - fw - 1
-                    + d_data->layoutData.scale[axis].end );
+                if ( axis == QwtPlot::xTop )
+                    sRect.setBottom( canvasRect.top() );
+                else
+                    sRect.setTop( canvasRect.bottom() );
             }
         }
+        else
+        {
+            if ( d_data->alignCanvasToScales[QwtPlot::xTop] )
+            {
+                double x = canvasRect.top() - d_data->layoutData.scale[axis].start;
+                if ( !( options & IgnoreFrames ) )
+                    x += d_data->layoutData.canvas.contentsMargins[ QwtPlot::xTop ];
 
-        if ( scaleRect[QwtPlot::xTop].isValid() )
-            scaleRect[QwtPlot::xTop].setBottom( canvasRect.top() );
-        if ( scaleRect[QwtPlot::xBottom].isValid() )
-            scaleRect[QwtPlot::xBottom].setTop( canvasRect.bottom() );
-        if ( scaleRect[QwtPlot::yLeft].isValid() )
-            scaleRect[QwtPlot::yLeft].setRight( canvasRect.left() );
-        if ( scaleRect[QwtPlot::yRight].isValid() )
-            scaleRect[QwtPlot::yRight].setLeft( canvasRect.right() );
+                sRect.setTop( x );
+            }
+            if ( d_data->alignCanvasToScales[QwtPlot::xBottom] )
+            {
+                double x = canvasRect.bottom() - 1 + d_data->layoutData.scale[axis].end;
+                if ( !( options & IgnoreFrames ) )
+                    x -= d_data->layoutData.canvas.contentsMargins[ QwtPlot::xBottom ];
+
+                sRect.setBottom( x );
+            }
+
+            if ( d_data->alignCanvasToScales[ axis ] )
+            {
+                if ( axis == QwtPlot::yLeft )
+                    sRect.setRight( canvasRect.left() );
+                else
+                    sRect.setLeft( canvasRect.right() );
+            }
+        }
     }
 }
 
@@ -1059,10 +1261,10 @@ void QwtPlotLayout::alignScales( int options,
   \brief Recalculate the geometry of all components.
 
   \param plot Plot to be layout
-  \param plotRect Rect where to place the components
+  \param plotRect Rectangle where to place the components
   \param options Layout options
 
-  \sa invalidate(), titleRect(),
+  \sa invalidate(), titleRect(), footerRect()
       legendRect(), scaleRect(), canvasRect()
 */
 void QwtPlotLayout::activate( const QwtPlot *plot,
@@ -1072,13 +1274,12 @@ void QwtPlotLayout::activate( const QwtPlot *plot,
 
     QRectF rect( plotRect );  // undistributed rest of the plot rect
 
-    // We extract all layout relevant data from the widgets,
-    // filter them through pfilter and save them to d_data->layoutData.
+    // We extract all layout relevant parameters from the widgets,
+    // and save them to d_data->layoutData.
 
     d_data->layoutData.init( plot, rect );
 
     if ( !( options & IgnoreLegend )
-        && d_data->legendPos != QwtPlot::ExternalLegend
         && plot->legend() && !plot->legend()->isEmpty() )
     {
         d_data->legendRect = layoutLegend( options, rect );
@@ -1086,7 +1287,7 @@ void QwtPlotLayout::activate( const QwtPlot *plot,
         // subtract d_data->legendRect from rect
 
         const QRegion region( rect.toRect() );
-        rect = region.subtract( d_data->legendRect.toRect() ).boundingRect();
+        rect = region.subtracted( d_data->legendRect.toRect() ).boundingRect();
 
         switch ( d_data->legendPos )
         {
@@ -1102,8 +1303,6 @@ void QwtPlotLayout::activate( const QwtPlot *plot,
             case QwtPlot::BottomLegend:
                 rect.setBottom( rect.bottom() - d_data->spacing );
                 break;
-            case QwtPlot::ExternalLegend:
-                break; // suppress compiler warning
         }
     }
 
@@ -1120,23 +1319,27 @@ void QwtPlotLayout::activate( const QwtPlot *plot,
      +---+-----------+---+
      |   |   Axis    |   |
      +---+-----------+---+
+     |      Footer       |
+     +---+-----------+---+
     */
 
-    // axes and title include text labels. The height of each
+    // title, footer and axes include text labels. The height of each
     // label depends on its line breaks, that depend on the width
     // for the label. A line break in a horizontal text will reduce
     // the available width for vertical texts and vice versa.
-    // expandLineBreaks finds the height/width for title and axes
+    // expandLineBreaks finds the height/width for title, footer and axes
     // including all line breaks.
 
-    int dimTitle, dimAxes[QwtPlot::axisCnt];
-    expandLineBreaks( options, rect, dimTitle, dimAxes );
+    int dimTitle, dimFooter, dimAxes[QwtPlot::axisCnt];
+    expandLineBreaks( options, rect, dimTitle, dimFooter, dimAxes );
 
     if ( dimTitle > 0 )
     {
-        d_data->titleRect.setRect( 
+        d_data->titleRect.setRect(
             rect.left(), rect.top(), rect.width(), dimTitle );
 
+        rect.setTop( d_data->titleRect.bottom() + d_data->spacing );
+
         if ( d_data->layoutData.scale[QwtPlot::yLeft].isEnabled !=
             d_data->layoutData.scale[QwtPlot::yRight].isEnabled )
         {
@@ -1147,9 +1350,25 @@ void QwtPlotLayout::activate( const QwtPlot *plot,
             d_data->titleRect.setWidth( rect.width()
                 - dimAxes[QwtPlot::yLeft] - dimAxes[QwtPlot::yRight] );
         }
+    }
 
-        // subtract title
-        rect.setTop( rect.top() + dimTitle + d_data->spacing );
+    if ( dimFooter > 0 )
+    {
+        d_data->footerRect.setRect(
+            rect.left(), rect.bottom() - dimFooter, rect.width(), dimFooter );
+
+        rect.setBottom( d_data->footerRect.top() - d_data->spacing );
+
+        if ( d_data->layoutData.scale[QwtPlot::yLeft].isEnabled !=
+            d_data->layoutData.scale[QwtPlot::yRight].isEnabled )
+        {
+            // if only one of the y axes is missing we align
+            // the footer centered to the canvas
+
+            d_data->footerRect.setX( rect.left() + dimAxes[QwtPlot::yLeft] );
+            d_data->footerRect.setWidth( rect.width()
+                - dimAxes[QwtPlot::yLeft] - dimAxes[QwtPlot::yRight] );
+        }
     }
 
     d_data->canvasRect.setRect(
