@@ -216,10 +216,21 @@ void QwtPlotTextLabel::draw( QPainter *painter,
         pixmapRect.setRight( qCeil( rect.right() ) + pw );
         pixmapRect.setBottom( qCeil( rect.bottom() ) + pw );
         
+#define QWT_HIGH_DPI 1
+
+#if QT_VERSION >= 0x050100 && QWT_HIGH_DPI
+        const qreal pixelRatio = painter->device()->devicePixelRatio();
+        const QSize scaledSize = pixmapRect.size() * pixelRatio;
+#else
+        const QSize scaledSize = pixmapRect.size();
+#endif
         if ( d_data->pixmap.isNull() || 
-            ( pixmapRect.size() != d_data->pixmap.size() )  )
+            ( scaledSize != d_data->pixmap.size() )  )
         {
-            d_data->pixmap = QPixmap( pixmapRect.size() );
+            d_data->pixmap = QPixmap( scaledSize );
+#if QT_VERSION >= 0x050100 && QWT_HIGH_DPI
+            d_data->pixmap.setDevicePixelRatio( pixelRatio );
+#endif
             d_data->pixmap.fill( Qt::transparent );
 
             const QRect r( pw, pw, 
